
# q-expansion of newform 4032.2.a.h, downloaded from the LMFDB on 20 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4032
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-2], [-1], [4], [2], [6], [-4], [0], [-2], [0], [-6], [-2], [4], [0], [6], [12], [2], [-4], [0], [-6], [-16], [-12], [14], [18], [14], [8], [4], [18], [14], [0], [4], [6], [-12], [6], [8], [2], [-4], [8], [-10], [-4], [26], [8], [2], [22], [24], [-4], [16], [-12], [10], [6], [-24], [2], [-20], [-26], [-16], [6], [16], [-22], [22], [20], [14], [-4], [24], [26], [-18], [4], [-14], [-28], [2], [-10], [-32], [0], [10], [-12], [0], [6], [18], [30], [-22], [-12], [-38], [24], [-14], [-24], [36], [30], [10], [-10], [16], [36], [16], [-8], [20], [-4], [-24], [-10], [-18], [20], [34], [-4], [-2], [4], [-10], [4], [34], [28], [6], [-48], [-6], [-16], [26], [6], [20], [-40], [-18], [-20], [40], [-18], [12], [-22], [34], [-18], [-12], [-20], [30], [-6], [48], [-40], [18], [-36], [0], [-32], [10], [-18], [2], [14], [44], [-26], [-42], [-44], [38], [24], [-12], [-14], [8], [10], [14], [-44], [24], [-46], [6], [28], [-8], [4], [24], [8], [-26], [42], [38], [44], [-26], [40], [12], [30], [-24], [-16], [26], [-14], [-34], [-12], [-30], [0], [-22], [32], [-2], [-28], [-26], [-24], [-46], [32], [-60], [-38], [46], [-24], [-34], [2], [-4], [10], [32], [-14], [-60], [28], [6], [28], [-18], [50], [-30], [6], [-24], [-34], [32], [-6], [-14], [44], [-58], [-48], [-36], [-42], [-28], [18], [-18], [-56], [28], [40], [10], [-64], [38], [32], [30], [-38], [-8], [-2], [32], [20], [10], [14], [-8], [8], [-12], [-46], [-4], [-16], [-10], [36], [-48], [-62], [38], [36], [-8], [60], [36], [-72], [-46], [54], [-40], [-32], [20], [20], [-24], [34], [-42], [-64], [58], [22], [44], [10], [-12], [-18], [26], [32], [-52], [10], [34], [22], [28], [-66], [62], [-28], [70], [50], [-68], [10], [-16], [-62], [56], [52], [-30], [10], [68], [64], [-24], [40], [-70], [-44], [24], [82], [6], [56], [6], [-2], [52], [54], [60], [50], [-26], [-80], [14], [-52], [-68], [-54], [-18], [-80], [-20], [-60], [66], [-12], [-30], [80], [26], [48], [-10], [-18], [12], [16], [26], [-12], [24], [-14], [-66], [28], [-38], [6], [-16], [-42], [34], [-4], [-44], [-56], [6], [50], [30], [32], [-12], [20], [-20], [34], [6], [-70], [16], [74], [22], [30], [-8], [-34], [28], [74], [-28], [-48], [-34], [-52], [-54], [-2], [96], [-22], [-2], [0], [12], [46], [48], [-6], [62], [24], [-76], [28], [-86], [-58], [40], [90], [12], [36], [-72], [-90], [56], [2], [52], [-40], [2], [-90], [-54], [38], [54], [40], [-2], [60], [-8], [32], [-70], [4], [48], [34], [-66], [36], [28], [-80], [58], [16], [-18], [-12], [-42], [34], [94], [-80], [78], [30], [-8], [18], [-50], [76], [84], [50], [86], [-28], [-52], [-54], [6], [24], [-88], [-90], [-72], [-66], [-70], [-8], [-100], [74], [-42], [28], [54], [36], [24]]
